@@ -15,7 +15,6 @@ import (
 	"crypto/tls"
 	"fmt"
 	"hash"
-	"internal/synctest"
 	"io"
 	"log"
 	"maps"
@@ -34,6 +33,7 @@ import (
 	"sync"
 	"sync/atomic"
 	"testing"
+	"testing/synctest"
 	"time"
 )
 
@@ -95,33 +95,13 @@ func run[T TBRun[T]](t T, f func(t T, mode testMode), opts ...any) {
 	}
 }
 
-// cleanupT wraps a testing.T and adds its own Cleanup method.
-// Used to execute cleanup functions within a synctest bubble.
-type cleanupT struct {
-	*testing.T
-	cleanups []func()
-}
-
-// Cleanup replaces T.Cleanup.
-func (t *cleanupT) Cleanup(f func()) {
-	t.cleanups = append(t.cleanups, f)
-}
-
-func (t *cleanupT) done() {
-	for _, f := range slices.Backward(t.cleanups) {
-		f()
-	}
-}
-
 // runSynctest is run combined with synctest.Run.
 //
 // The TB passed to f arranges for cleanup functions to be run in the synctest bubble.
-func runSynctest(t *testing.T, f func(t testing.TB, mode testMode), opts ...any) {
+func runSynctest(t *testing.T, f func(t *testing.T, mode testMode), opts ...any) {
 	run(t, func(t *testing.T, mode testMode) {
-		synctest.Run(func() {
-			ct := &cleanupT{T: t}
-			defer ct.done()
-			f(ct, mode)
+		synctest.Test(t, func(t *testing.T) {
+			f(t, mode)
 		})
 	}, opts...)
 }
@@ -292,12 +272,12 @@ func TestNewClientServerTest(t *testing.T) {
 		}, modes)
 	})
 	t.Run("synctest", func(t *testing.T) {
-		runSynctest(t, func(t testing.TB, mode testMode) {
+		runSynctest(t, func(t *testing.T, mode testMode) {
 			testNewClientServerTest(t, mode, optFakeNet)
 		}, modes)
 	})
 }
-func testNewClientServerTest(t testing.TB, mode testMode, opts ...any) {
+func testNewClientServerTest(t *testing.T, mode testMode, opts ...any) {
 	var got struct {
 		sync.Mutex
 		proto  string
@@ -1253,7 +1233,7 @@ func testTransportGCRequest(t *testing.T, mode testMode, body bool) {
 	(func() {
 		body := strings.NewReader("some body")
 		req, _ := NewRequest("POST", cst.ts.URL, body)
-		runtime.SetFinalizer(req, func(*Request) { close(didGC) })
+		runtime.AddCleanup(req, func(ch chan struct{}) { close(ch) }, didGC)
 		res, err := cst.c.Do(req)
 		if err != nil {
 			t.Fatal(err)
